package inf1010.pusle.pathfinding.graph;

import java.util.Collection;
import java.util.Map;
import java.util.HashMap;

/**
 * Represents a vertex in a graph.
 * It contains a map of all edges between this vertex and its neigbours.
 */
public class Vertex implements Comparable<Vertex> {

    /** The name of this vertex (e.g. name of a city). */
    private String name;

    /** "Latitude and longitude", basically any kind of 2D coordinates. */
    private double latitude, longitude;

    /** A map containing all edges to neighbouring vertices. */
    private Map<Vertex, Edge> neighbours = new HashMap<Vertex, Edge>();

    /**
     * Creates a new vertex.
     *
     * @param name The name of the vertex.
     * @param latitude The latitude of the vertex.
     * @param longitude The longitude of the vertex.
     */
    public Vertex(String name, double latitude, double longitude) {
        this.name = name;
        this.latitude = latitude;
        this.longitude = longitude;
    }

    /**
     * Gets this vertex's name.
     *
     * @return This vertex's name.
     */
    public String getName() {
        return name;
    }

    /**
     * Adds an edge to the map of neighbours.
     *
     * It does nothing, if either argument is null,
     * the neighbour is the same as this object,
     * or the neighbour or this node isn't either A or B in the edge.
     *
     * @param e The edge between the edges.
     * @param neighbour The neighbouring vertex.
     */
    public void addEdge(Edge e, Vertex neighbour) {
        if (e == null
            || neighbour == null
            || neighbour.equals(this)
            || !(neighbour.equals(e.getA()) || neighbour.equals(e.getB()))
            || !(this.equals(e.getA()) || this.equals(e.getB()))) {
            return;
        }

        neighbours.put(neighbour, e);
    }

    /**
     * Gets all neighbouring vertices.
     *
     * @return A collection with all neighbouring vertices.
     */
    public Collection<Vertex> getNeighbours() {
        return neighbours.keySet();
    }

    /**
     * Gets the distance to a neighbouring vertex.
     *
     * @param v The neighbour to get the distance to.
     * @return The distance to the vertex or -1 if it's not a neighbour.
     */
    public double getDistance(Vertex v) {
        Edge e = neighbours.get(v);
        if (e == null)
            return -1;

        return e.getDistance();
    }

    /**
     * {@inheritDoc}
     */
    public int hashCode() {
        return name.hashCode();
    }

    /**
     * {@inheritDoc}
     */
    public boolean equals(Object o) {
        if (o instanceof Vertex) {
            Vertex v = (Vertex) o;
            if (name.equals(v.name)
                && latitude == v.latitude
                && longitude == v.longitude) {
                return true;
            }
        }

        return false;
    }

    /**
     * Compares this vertex's name to another vertex's name.
     *
     * @param v The vertex to compare this to.
     * @return {@inheritDoc}
     */
    public int compareTo(Vertex v) {
        return name.compareTo(v.name);
    }
}